indexing
   description: "Create the color palette and the gradient"
   status: "Free according to the GPL license (see COPYING)"
   date: "$Date$"
   revision: "$Revision$"
   author: "Mark Arrasmith <arrasmith@math.twsu.edu> and Miron Brezuleanu"
   
class COLORS

inherit
   EXG_TIME
   EXG_STORABLE
   STARTUP
   
creation
   make
   
feature
   
   make is
      do
	 build_palette
	 --num_colors := config.colors
      end
   
   build_palette is
      local
	 color: GDK_COLOR
      do
	 !ARRAY[GDK_COLOR]!palette.make(1, 0)
	 !!color.make(0, 0, 0)
	 palette.add_last(color)
	 !!color.make(30720, 44032, 20480)
	 palette.add_last(color)
	 !!color.make(34816, 26624, 20480)
	 palette.add_last(color)
	 !!color.make(8192, 25600, 32768)
	 palette.add_last(color)
	 !!color.make(32768, 35840, 34816)
	 palette.add_last(color)
	 !!color.make(26624, 0, 0)
	 palette.add_last(color)
	 !!color.make(45056, 36864, 16384)
	 palette.add_last(color)
	 !!color.make(45056, 48128, 8192)
	 palette.add_last(color)
	 !!color.make(16384, 16384, 16384)
	 palette.add_last(color)
	 !!color.make(0, 19456, 0)
	 palette.add_last(color)
	 !!color.make(24576, 15360, 34816)
	 palette.add_last(color)
	 !!color.make(4096, 27648, 28672)
	 palette.add_last(color)
	 
      end

     reset is
	 -- will reset the `colors' component
      do
	 build_palette
      end

   save_to_file(file: STD_FILE_WRITE) is
	 -- like `load_from_file' but does the saving
      local
	 i: INTEGER
      do
	 from
	    i := palette.lower
	 until
	    i > palette.upper
	 loop
	    file.put_string(palette.item(i).red.to_string + " ")
	    file.put_string(palette.item(i).green.to_string + " ")
	    file.put_string(palette.item(i).blue.to_string + "%N")
	    i := i + 1
	 end
	 file.put_string("******%N")
      end      
      
    load_from_file(file: STD_FILE_READ) is
	 -- will load the colors from the given file
      local
	 stage: INTEGER	 
	 components: ARRAY[STRING]
	 color: GDK_COLOR
      do
	 from
	    stage := 0
	    !ARRAY[GDK_COLOR]!palette.make(1, 0)
	 until
	    file.end_of_input or stage = 2
	 loop
	    file.read_line
	    inspect stage
	    when 0 then
	       if equal(file.last_string, "******") then
		  stage := 1
	       else
		  components := file.last_string.split
		  !!color.make(components.item(1).to_integer,
			       components.item(2).to_integer,
			       components.item(3).to_integer)  
		  palette.add_last(color)
	       end
	    when 1 then
	       components := file.last_string.split
	       stage := 2
	    end
	 end
      end
  
   signature: STRING is
      do
	 Result := "efractal color map"
      end
  
   gradient: COLLECTION[GDK_COLOR] is
	 -- will build the gradient based on the `palette'; the
	 -- gradient will have `num_colors' entries;
      local
	 lengths: ARRAY[DOUBLE]
	 length: DOUBLE 
      do
	 palette.put(palette.item(1), 1)
	 palette.put(palette.item(2), 2)
	 palette.put(palette.item(3), 3)
	 palette.put(palette.item(4), 4)
	 palette.put(palette.item(5), 5)
	 palette.put(palette.item(6), 6)
	 palette.put(palette.item(7), 7)
	 palette.put(palette.item(8), 8)
	 palette.put(palette.item(9), 9)
	 palette.put(palette.item(10), 10)
	 palette.put(palette.item(11), 11)
	 palette.put(palette.item(12), 12)
	 !!lengths.make(1, 12)

	 lengths.put((sqr((palette.item(1).red - palette.item(2).red).to_double) +
		      sqr((palette.item(1).green - palette.item(2).green).to_double) +
		      sqr((palette.item(1).blue - palette.item(2).blue).to_double)).sqrt
		     , 1)
	    
	 lengths.put((sqr((palette.item(2).red - palette.item(3).red).to_double) + 
		      sqr((palette.item(2).green - palette.item(3).green).to_double) +
		      sqr((palette.item(2).blue - palette.item(3).blue).to_double)).sqrt
		     , 2)
	    
	 lengths.put((sqr((palette.item(3).red - palette.item(4).red).to_double) +
		      sqr((palette.item(3).green - palette.item(4).green).to_double) +
		      sqr((palette.item(3).blue - palette.item(4).blue).to_double)).sqrt
		     , 3)

	 lengths.put((sqr((palette.item(4).red - palette.item(5).red).to_double) +
		      sqr((palette.item(4).green - palette.item(5).green).to_double) +
		      sqr((palette.item(4).blue - palette.item(5).blue).to_double)).sqrt
		     , 4)

	 lengths.put((sqr((palette.item(5).red - palette.item(6).red).to_double) +
		      sqr((palette.item(5).green - palette.item(6).green).to_double) +
		      sqr((palette.item(5).blue - palette.item(6).blue).to_double)).sqrt
		     , 5)

	 lengths.put((sqr((palette.item(6).red - palette.item(7).red).to_double) +
		      sqr((palette.item(6).green - palette.item(7).green).to_double) +
		      sqr((palette.item(6).blue - palette.item(7).blue).to_double)).sqrt
		     , 6)

	 lengths.put((sqr((palette.item(7).red - palette.item(8).red).to_double) +
		      sqr((palette.item(7).green - palette.item(8).green).to_double) +
		      sqr((palette.item(7).blue - palette.item(8).blue).to_double)).sqrt
		     , 7)

	 lengths.put((sqr((palette.item(8).red - palette.item(9).red).to_double) +
		      sqr((palette.item(8).green - palette.item(9).green).to_double) +
		      sqr((palette.item(8).blue - palette.item(9).blue).to_double)).sqrt
		     , 8)

	 lengths.put((sqr((palette.item(9).red - palette.item(10).red).to_double) +
		      sqr((palette.item(9).green - palette.item(10).green).to_double) +
		      sqr((palette.item(9).blue - palette.item(10).blue).to_double)).sqrt
		     , 9)

	 lengths.put((sqr((palette.item(10).red - palette.item(11).red).to_double) +
		      sqr((palette.item(10).green - palette.item(11).green).to_double) +
		      sqr((palette.item(10).blue - palette.item(11).blue).to_double)).sqrt
		     , 10)

	 lengths.put((sqr((palette.item(11).red - palette.item(12).red).to_double) +
		      sqr((palette.item(11).green - palette.item(12).green).to_double) +
		      sqr((palette.item(11).blue - palette.item(12).blue).to_double)).sqrt
		     , 11)

	 lengths.put((sqr((palette.item(12).red - palette.item(1).red).to_double) +
		      sqr((palette.item(12).green - palette.item(1).green).to_double) +
		      sqr((palette.item(12).blue - palette.item(1).blue).to_double)).sqrt
		     , 12)
	    
	 length := (lengths.item(1)+lengths.item(2)+lengths.item(3)+
		    lengths.item(4)+lengths.item(5)+lengths.item(6)+
		    lengths.item(7)+lengths.item(8)+lengths.item(9)+
		    lengths.item(10)+lengths.item(11)+lengths.item(12))
	 !ARRAY[GDK_COLOR]!Result.make(1, num_colors)
	 last_index := 1
	 interpolate(Result, palette.item(1), palette.item(2),
		     (num_colors.to_double * (lengths.item(1) / length)).floor)
	 interpolate(Result, palette.item(2), palette.item(3),
		     (num_colors.to_double * (lengths.item(2) / length)).floor)
	 interpolate(Result, palette.item(3), palette.item(4), 
		     (num_colors.to_double * (lengths.item(3) / length)).floor)
	 interpolate(Result, palette.item(4), palette.item(5), 
		     (num_colors.to_double * (lengths.item(4) / length)).floor)
	 interpolate(Result, palette.item(5), palette.item(6), 
		     (num_colors.to_double * (lengths.item(5) / length)).floor)
	 interpolate(Result, palette.item(6), palette.item(7), 
		     (num_colors.to_double * (lengths.item(6) / length)).floor)
	 interpolate(Result, palette.item(7), palette.item(8), 
		     (num_colors.to_double * (lengths.item(7) / length)).floor)
	 interpolate(Result, palette.item(8), palette.item(9), 
		     (num_colors.to_double * (lengths.item(8) / length)).floor)
	 interpolate(Result, palette.item(9), palette.item(10), 
		     (num_colors.to_double * (lengths.item(9) / length)).floor)
	 interpolate(Result, palette.item(10), palette.item(11), 
		     (num_colors.to_double * (lengths.item(10) / length)).floor)
	 interpolate(Result, palette.item(11), palette.item(12), 
		     (num_colors.to_double * (lengths.item(11) / length)).floor)
	 interpolate(Result, palette.item(12), palette.item(1), 
		     (num_colors.to_double * (lengths.item(12) / length)).floor)
	 from
	 until
	    last_index > num_colors
	 loop
	    Result.put(last_color, last_index)
	    last_index := last_index + 1
	 end
      end
   
   last_index: INTEGER
   last_color: GDK_COLOR
	 -- values used for communication between `build_gradient' and
	 -- `interpolate'
   
   interpolate(pgradient: COLLECTION[GDK_COLOR];
	       c1, c2: GDK_COLOR; 
	       num_colors_in_between: INTEGER) is
	 -- will create `num_colors' colors between `c1' and `c2' and will 
	 -- place them in `pgradient'
      local
	 i: INTEGER
	 red, green, blue: DOUBLE
	 -- these are the initial values
	 
	 r_i, g_i, b_i: DOUBLE
	 -- these are the increments
      do
	 red := c1.red.to_double
	 green := c1.green.to_double
	 blue := c1.blue.to_double
	 r_i := (c2.red.to_double - red) / num_colors_in_between.to_double
	 g_i := (c2.green.to_double - green) / num_colors_in_between.to_double
	 b_i := (c2.blue.to_double - blue) / num_colors_in_between.to_double
	 from
	    i := 1
	 until
	    i > num_colors_in_between
	 loop
	    !!last_color.make((red + i.to_double * r_i).rounded,
			      (green + i.to_double * g_i).rounded,
			      (blue + i.to_double * b_i).rounded)
	    pgradient.put(last_color, last_index)
	    last_index := last_index + 1
	    i := i + 1
	 end
      end
   
   sqr(v: DOUBLE): DOUBLE is
	 -- why is this missing from double.e?
      do
	 Result := v * v
      end
   
   set_num_colors(nv: INTEGER) is
	 -- setter for `num_colors'
      do
	 --num_colors := nv
	 config.set_colors(nv)
      end
   
feature -- attributes
   
   palette: COLLECTION[GDK_COLOR]
	 -- the palette of colors we'll select the three gradient 
	 -- generators from
   
   num_colors: INTEGER is 
     do
	 Result := config.colors
	 end
	 -- the number of colors in the gradient
   
end
   
